AQUALINK_API_KEY = "EOOEMOW4YR6QNB07"

AQUALINK_LOGIN_URL = "https://support.iaqualink.com/users/sign_in.json"
AQUALINK_DEVICES_URL = "https://support.iaqualink.com/devices.json"
AQUALINK_SESSION_URL = "https://iaqualink-api.realtime.io/v1/mobile/session.json"

AQUALINK_TEMP_CELSIUS_LOW = 1
AQUALINK_TEMP_CELSIUS_HIGH = 40
AQUALINK_TEMP_FAHRENHEIT_LOW = 32
AQUALINK_TEMP_FAHRENHEIT_HIGH = 104



import logging
import time
import traceback

import aiohttp

from iaqualink.device import AqualinkDevice
from iaqualink.exception import AqualinkSystemOfflineException
from iaqualink.typing import Payload

MIN_SECS_TO_REFRESH = 15

LOGGER = logging.getLogger("iaqualink")


class AqualinkSystem(object):
    def __init__(self, aqualink: "AqualinkClient", data: "Payload"):
        self.aqualink = aqualink
        self.data = data
        self.devices = {}
        self.has_spa = None
        self.temp_unit = None
        self.last_refresh = 0
        self.last_run_success = None
        self.online = None

    def __repr__(self) -> str:
        attrs = ["name", "serial", "data"]
        attrs = ["%s=%r" % (i, getattr(self, i)) for i in attrs]
        return f'{self.__class__.__name__}({" ".join(attrs)})'

    @property
    def name(self) -> str:
        return self.data["name"]

    @property
    def serial(self) -> str:
        return self.data["serial_number"]

    @classmethod
    def from_data(cls, aqualink: "AqualinkClient", data: "Payload"):
        SYSTEM_TYPES = {"iaqua": AqualinkPoolSystem}

        class_ = SYSTEM_TYPES.get(data["device_type"])

        if class_ is None:
            LOGGER.warning(f"{data['device_type']} is not a supported system type.")
            return None

        return class_(aqualink, data)

    async def get_devices(self):
        if not self.devices:
            await self.update()
        return self.devices

    async def update(self) -> None:
        # Be nice to Aqualink servers since we rely on polling.
        now = int(time.time())
        delta = now - self.last_refresh
        if delta < MIN_SECS_TO_REFRESH:
            LOGGER.debug(f"Only {delta}s since last refresh.")
            return

        try:
            r1 = await self.aqualink.send_home_screen_request(self.serial)
            r2 = await self.aqualink.send_devices_screen_request(self.serial)
            await self._parse_home_response(r1)
            await self._parse_devices_response(r2)
        except AqualinkSystemOfflineException:
            self.last_run_success = True
            self.online = False
        except Exception as e:  # pylint: disable=W0703
            self.last_run_success = False
            self.online = None
            LOGGER.error(f"Unhandled exception: {e}")
            for line in traceback.format_exc().split("\n"):
                LOGGER.error(line)
        else:
            self.last_run_success = True
            self.online = True
            self.last_refresh = int(time.time())

    async def _parse_home_response(self, response: aiohttp.ClientResponse) -> None:
        data = await response.json()

        if data["home_screen"][0]["status"] == "Offline":
            LOGGER.warning(f"Status for system {self.serial} is Offline.")
            raise AqualinkSystemOfflineException

        self.temp_unit = data["home_screen"][3]["temp_scale"]

        # Make the data a bit flatter.
        devices = {}
        for x in data["home_screen"][4:]:
            name = list(x.keys())[0]
            state = list(x.values())[0]
            attrs = {"name": name, "state": state}
            devices.update({name: attrs})

        for k, v in devices.items():
            if k in self.devices:
                for dk, dv in v.items():
                    self.devices[k].data[dk] = dv
            else:
                self.devices[k] = AqualinkDevice.from_data(self, v)

        # Keep track of the presence of the spa so we know whether temp1 is
        # for the spa or the pool. This is pretty ugly.
        if "spa_set_point" in devices:
            self.has_spa = True
        else:
            self.has_spa = False

    async def _parse_devices_response(self, response: aiohttp.ClientResponse) -> None:
        data = await response.json()

        if data["devices_screen"][0]["status"] == "Offline":
            LOGGER.warning(f"Status for system {self.serial} is Offline.")
            raise AqualinkSystemOfflineException

        # Make the data a bit flatter.
        devices = {}
        for i, x in enumerate(data["devices_screen"][3:], 1):
            attrs = {"aux": f"{i}", "name": list(x.keys())[0]}
            for y in list(x.values())[0]:
                attrs.update(y)
            devices.update({f"aux_{i}": attrs})

        for k, v in devices.items():
            if k in self.devices:
                for dk, dv in v.items():
                    self.devices[k].data[dk] = dv
            else:
                self.devices[k] = AqualinkDevice.from_data(self, v)

    async def set_pump(self, command: str) -> None:
        r = await self.aqualink.set_pump(self.serial, command)
        await self._parse_home_response(r)

    async def set_heater(self, command: str) -> None:
        r = await self.aqualink.set_heater(self.serial, command)
        await self._parse_home_response(r)

    async def set_temps(self, temps: Payload) -> None:
        r = await self.aqualink.set_temps(self.serial, temps)
        await self._parse_home_response(r)

    async def set_aux(self, aux: str) -> None:
        r = await self.aqualink.set_aux(self.serial, aux)
        await self._parse_devices_response(r)

    async def set_light(self, data: Payload) -> None:
        r = await self.aqualink.set_light(self.serial, data)
        await self._parse_devices_response(r)


class AqualinkPoolSystem(AqualinkSystem):
    pass



__author__ = "Florent Thoumie"
__email__ = "florent@thoumie.net"
__version__ = "0.3.1"

from .client import AqualinkClient, AqualinkLoginException
from .system import AqualinkSystem, AqualinkPoolSystem
from .device import (
    AqualinkAuxToggle,
    AqualinkBinarySensor,
    AqualinkColorLight,
    AqualinkDevice,
    AqualinkDimmableLight,
    AqualinkHeater,
    AqualinkLight,
    AqualinkLightEffect,
    AqualinkLightToggle,
    AqualinkLightType,
    AqualinkPump,
    AqualinkSensor,
    AqualinkState,
    AqualinkThermostat,
    AqualinkToggle,
)



import logging
import threading
from typing import Dict, Optional

import aiohttp

from iaqualink.const import (
    AQUALINK_API_KEY,
    AQUALINK_LOGIN_URL,
    AQUALINK_DEVICES_URL,
    AQUALINK_SESSION_URL,
)
from iaqualink.exception import AqualinkLoginException
from iaqualink.system import AqualinkSystem
from iaqualink.typing import Payload


AQUALINK_COMMAND_GET_DEVICES = "get_devices"
AQUALINK_COMMAND_GET_HOME = "get_home"
AQUALINK_COMMAND_GET_ONETOUCH = "get_onetouch"
AQUALINK_COMMAND_SET_AUX = "set_aux"
AQUALINK_COMMAND_SET_LIGHT = "set_light"
AQUALINK_COMMAND_SET_POOL_HEATER = "set_pool_heater"
AQUALINK_COMMAND_SET_POOL_PUMP = "set_pool_pump"
AQUALINK_COMMAND_SET_SOLAR_HEATER = "set_solar_heater"
AQUALINK_COMMAND_SET_SPA_HEATER = "set_spa_heater"
AQUALINK_COMMAND_SET_SPA_PUMP = "set_spa_pump"
AQUALINK_COMMAND_SET_TEMPS = "set_temps"

AQUALINK_HTTP_HEADERS = {
    "User-Agent": "iAquaLink/70 CFNetwork/901.1 Darwin/17.6.0",
    "Content-Type": "application/json",
    "Accept": "*/*",
}

LOGGER = logging.getLogger("aqualink")


class AqualinkClient(object):
    def __init__(
        self,
        username: str,
        password: str,
        session: Optional[aiohttp.ClientSession] = None,
    ):
        self.username = username
        self.password = password

        if session is None:
            self.session = aiohttp.ClientSession()
            self._must_clean_session = True
        else:
            self.session = session
            self._must_clean_session = False

        self.session_id = None
        self.token = None
        self.user_id = None

        self.lock = threading.Lock()
        self.last_refresh = 0

    async def __aenter__(self):
        await self.login()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        # All Exceptions get re-raised.
        if self._must_clean_session is True:
            await self.session.close()
        return exc is None

    async def _send_request(
        self, url: str, method: str = "get", **kwargs
    ) -> aiohttp.ClientResponse:
        LOGGER.debug(f"-> {method.upper()} {url} {kwargs}")
        r = await self.session.request(
            method, url, headers=AQUALINK_HTTP_HEADERS, **kwargs
        )
        if r.status == 200:
            LOGGER.debug(f"<- {r.status} {r.reason} - {url}")
        else:
            LOGGER.warning(f"<- {r.status} {r.reason} - {url}")
        return r

    async def _send_login_request(self) -> aiohttp.ClientResponse:
        data = {
            "api_key": AQUALINK_API_KEY,
            "email": self.username,
            "password": self.password,
        }
        return await self._send_request(AQUALINK_LOGIN_URL, method="post", json=data)

    async def login(self) -> None:
        r = await self._send_login_request()

        if r.status == 200:
            data = await r.json()
            self.session_id = data["session_id"]
            self.token = data["authentication_token"]
            self.user_id = data["id"]
        else:
            raise AqualinkLoginException(f"Login failed: {r.status} {r.reason}")

    async def _send_systems_request(self) -> aiohttp.ClientResponse:
        params = {
            "api_key": AQUALINK_API_KEY,
            "authentication_token": self.token,
            "user_id": self.user_id,
        }
        params = "&".join(f"{k}={v}" for k, v in params.items())
        url = f"{AQUALINK_DEVICES_URL}?{params}"
        return await self._send_request(url)

    async def get_systems(self) -> Dict[str, "AqualinkSystems"]:
        r = await self._send_systems_request()

        if r.status == 200:
            data = await r.json()
            systems = [AqualinkSystem.from_data(self, x) for x in data]
            return {x.serial: x for x in systems if x is not None}

        raise Exception(f"Unable to retrieve systems list: {r.status} {r.reason}")

    async def _send_session_request(
        self, serial: str, command: str, params: Optional[Payload] = None
    ) -> aiohttp.ClientResponse:
        if not params:
            params = {}

        params.update(
            {
                "actionID": "command",
                "command": command,
                "serial": serial,
                "sessionID": self.session_id,
            }
        )
        params = "&".join(f"{k}={v}" for k, v in params.items())
        url = f"{AQUALINK_SESSION_URL}?{params}"
        return await self._send_request(url)

    async def send_home_screen_request(self, serial) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, AQUALINK_COMMAND_GET_HOME)
        return r

    async def send_devices_screen_request(self, serial) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, AQUALINK_COMMAND_GET_DEVICES)
        return r

    async def set_pump(self, serial: str, command: str) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, command)
        return r

    async def set_heater(self, serial: str, command: str) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, command)
        return r

    async def set_temps(self, serial: str, temps: Payload) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, AQUALINK_COMMAND_SET_TEMPS, temps)
        return r

    async def set_aux(self, serial: str, aux: str) -> aiohttp.ClientResponse:
        command = AQUALINK_COMMAND_SET_AUX + "_" + aux.replace("aux_", "")
        r = await self._send_session_request(serial, command)
        return r

    async def set_light(self, serial: str, data: Payload) -> aiohttp.ClientResponse:
        r = await self._send_session_request(serial, AQUALINK_COMMAND_SET_LIGHT, data)
        return r



from enum import Enum, auto, unique
import logging
from typing import Optional

from iaqualink.typing import DeviceData
from iaqualink.const import (
    AQUALINK_TEMP_CELSIUS_LOW,
    AQUALINK_TEMP_CELSIUS_HIGH,
    AQUALINK_TEMP_FAHRENHEIT_LOW,
    AQUALINK_TEMP_FAHRENHEIT_HIGH,
)


@unique
class AqualinkState(Enum):
    OFF = "0"
    ON = "1"
    ENABLED = "3"


# XXX - I don't know the exact values per type. The enum is pretty much a
# placeholder. If you know what type of lights you have and have debugging
# on, please submit an issue to GitHub with the details so I can update the
# code.
@unique
class AqualinkLightType(Enum):
    JANDY_LED_WATERCOLORS = auto()
    JANDY_COLORS = auto()
    HAYWARD_COLOR_LOGIC = auto()
    PENTAIR_INTELLIBRITE = auto()
    PENTAIR_SAM_SAL = auto()


# XXX - These values are probably LightType-specific but they're all I have
# at the moment. I can see this changing into a color profile system later.
class AqualinkLightEffect(Enum):
    NONE = "0"
    ALPINE_WHITE = "1"
    SKY_BLUE = "2"
    COBALT_BLUE = "3"
    CARIBBEAN_BLUE = "4"
    SPRING_GREEN = "5"
    EMERALD_GREEN = "6"
    EMERALD_ROSE = "7"
    MAGENTA = "8"
    VIOLENT = "9"
    SLOW_COLOR_SPLASH = "10"
    FAST_COLOR_SPLASH = "11"
    USA = "12"
    FAT_TUESDAY = "13"
    DISCO_TECH = "14"


LOGGER = logging.getLogger("aqualink")


class AqualinkDevice(object):
    def __init__(self, system: "AqualinkSystem", data: "DeviceData"):
        self.system = system
        self.data = data

    def __repr__(self) -> str:
        attrs = ["name", "data"]
        attrs = ["%s=%r" % (i, getattr(self, i)) for i in attrs]
        return f'{self.__class__.__name__}({" ".join(attrs)})'

    @property
    def label(self) -> str:
        if "label" in self.data:
            label = self.data["label"]
            return " ".join([x.capitalize() for x in label.split()])
        else:
            label = self.data["name"]
            return " ".join([x.capitalize() for x in label.split("_")])

    @property
    def state(self) -> str:
        return self.data["state"]

    @property
    def name(self) -> str:
        return self.data["name"]

    @classmethod
    def from_data(cls, system: "AqualinkSystem", data: DeviceData) -> "AqualinkDevice":
        if data["name"].endswith("_heater"):
            class_ = AqualinkHeater
        elif data["name"].endswith("_set_point"):
            class_ = AqualinkThermostat
        elif data["name"].endswith("_pump"):
            class_ = AqualinkPump
        elif data["name"] == "freeze_protection":
            class_ = AqualinkBinarySensor
        elif data["name"].startswith("aux_"):
            if data["type"] == "2":
                class_ = AqualinkColorLight
            elif data["type"] == "1":
                class_ = AqualinkDimmableLight
            elif "LIGHT" in data["label"]:
                class_ = AqualinkLightToggle
            else:
                class_ = AqualinkAuxToggle
        else:
            class_ = AqualinkSensor

        return class_(system, data)


class AqualinkSensor(AqualinkDevice):
    pass


class AqualinkBinarySensor(AqualinkSensor):
    """These are non-actionable sensors, essentially read-only on/off."""

    @property
    def is_on(self) -> bool:
        return (
            AqualinkState(self.state) in [AqualinkState.ON, AqualinkState.ENABLED]
            if self.state
            else False
        )


class AqualinkToggle(AqualinkDevice):
    @property
    def is_on(self) -> bool:
        return (
            AqualinkState(self.state) in [AqualinkState.ON, AqualinkState.ENABLED]
            if self.state
            else False
        )

    async def turn_on(self) -> None:
        if not self.is_on:
            await self.toggle()

    async def turn_off(self) -> None:
        if self.is_on:
            await self.toggle()

    async def toggle(self) -> None:
        raise NotImplementedError()


class AqualinkPump(AqualinkToggle):
    async def toggle(self) -> None:
        await self.system.set_pump(f"set_{self.name}")


class AqualinkHeater(AqualinkToggle):
    async def toggle(self) -> None:
        await self.system.set_heater(f"set_{self.name}")


class AqualinkAuxToggle(AqualinkToggle):
    async def toggle(self) -> None:
        await self.system.set_aux(self.data["aux"])


# Using AqualinkLight as a Mixin so we can use isinstance(dev, AqualinkLight).
class AqualinkLight(object):
    @property
    def brightness(self) -> Optional[int]:
        raise NotImplementedError()

    @property
    def effect(self) -> Optional[str]:
        raise NotImplementedError()

    @property
    def is_dimmer(self) -> bool:
        return self.brightness is not None

    @property
    def is_color(self) -> bool:
        return self.effect is not None


class AqualinkLightToggle(AqualinkLight, AqualinkAuxToggle):
    @property
    def brightness(self) -> Optional[bool]:
        return None

    @property
    def effect(self) -> Optional[int]:
        return None


class AqualinkDimmableLight(AqualinkLight, AqualinkDevice):
    @property
    def brightness(self) -> Optional[int]:
        return int(self.data["subtype"])

    @property
    def effect(self) -> Optional[int]:
        return None

    @property
    def is_on(self) -> bool:
        return self.brightness != 0

    async def set_brightness(self, brightness: int) -> None:
        # Brightness only works in 25% increments.
        if brightness not in [0, 25, 50, 75, 100]:
            msg = f"{brightness}% isn't a valid percentage. Only use 25% increments."
            raise Exception(msg)

        data = {"aux": self.data["aux"], "light": f"{brightness}"}
        await self.system.set_light(data)

    async def turn_on(self, level: int = 100) -> None:
        if self.brightness != level:
            await self.set_brightness(level)

    async def turn_off(self) -> None:
        if self.is_on:
            await self.set_brightness(0)


class AqualinkColorLight(AqualinkLight, AqualinkDevice):
    @property
    def brightness(self) -> Optional[int]:
        # Assuming that color lights don't have adjustable brightness.
        return None

    @property
    def effect(self) -> Optional[int]:
        return self.data["state"]

    @property
    def is_on(self) -> bool:
        return self.effect != "0"

    async def set_effect(self, effect: str) -> None:
        try:
            AqualinkLightEffect(effect)
        except Exception:
            msg = f"{repr(effect)} isn't a valid effect."
            raise Exception(msg)

        data = {
            "aux": self.data["aux"],
            "light": effect,
            "subtype": self.data["subtype"],
        }
        await self.system.set_light(data)

    async def turn_off(self):
        if self.is_on:
            await self.set_effect("0")

    async def turn_on(self):
        if not self.is_on:
            await self.set_effect("1")


class AqualinkThermostat(AqualinkDevice):
    @property
    def temp(self) -> str:
        # Spa takes precedence for temp1 if present.
        if self.name.startswith("pool") and self.system.has_spa:
            return "temp2"
        return "temp1"

    async def set_temperature(self, temperature: int) -> None:
        unit = self.system.temp_unit

        if unit == "F":
            low = AQUALINK_TEMP_FAHRENHEIT_LOW
            high = AQUALINK_TEMP_FAHRENHEIT_HIGH
        else:
            low = AQUALINK_TEMP_CELSIUS_LOW
            high = AQUALINK_TEMP_CELSIUS_HIGH

        if temperature not in range(low, high + 1):
            msg = f"{temperature}{unit} isn't a valid temperature"
            msg += f" ({low}-{high}{unit})."
            raise Exception(msg)

        data = {self.temp: temperature}
        await self.system.set_temps(data)



#!/usr/bin/env python

from setuptools import setup

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = ["aiohttp"]

setup_requirements = ["pytest-runner"]

test_requirements = ["asynctest", "pytest", "pytest-cov", "pytest-asyncio"]

setup(
    name="iaqualink",
    version="0.3.1",
    description="Asynchronous library for Jandy iAqualink",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Florent Thoumie",
    author_email="florent@thoumie.net",
    url="https://github.com/flz/iaqualink-py",
    packages=["iaqualink"],
    package_dir={"iaqualink": "src/iaqualink"},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    keywords="iaqualink",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    test_suite="tests",
    tests_require=test_requirements,
    setup_requires=setup_requirements,
    python_requires=">=3.5",
)



class AqualinkException(Exception):
    """Base exception for iAqualink library."""


class AqualinkLoginException(AqualinkException):
    """Exception raised when failing to log in."""


class AqualinkSystemOfflineException(AqualinkException):
    """Exception raised when a system is offline."""
